<?php
namespace app\api\controller;

use think\Config;
use think\Controller;
use think\Loader;
use think\Request;

class Download extends Controller
{
    private $version;
    private $domain;
    private $license;
    private $return = [
        'status_code' => 400,
        'error'       => [
            'message' => null,
            'code'    => 100,
            'context' => 'Exception',
        ],
        'status_txt'  => 'Bad Request',
    ];

    private $update;

    public function index($version = null)
    {
        //请求信息缺少判断
        if (empty($version)) {
            $this->return['error']['message'] = 'Invalid version format';
            return json($this->return);
        }
        if (!$this->license = $this->request->get('license')) {
            $this->return['error']['message'] = 'Missing license info';
            $this->return['error']['code']    = 103;
            return json($this->return);
        }

        //版本判断
        $this->update = Loader::model('update')->getInfo($version);
        if (empty($this->update)) {
            $this->return['error']['message'] = 'Invalid version';
            $this->return['error']['code']    = 102;
            return json($this->return);
        }

        //授权格式判断
        $this->license = explode(':', $this->license);
        if (!isset($this->license[1])) {
            $this->return['error']['message'] = 'Invalid license key';
            $this->return['error']['code']    = 105;
            return json($this->return);
        }

        $domain = $this->request->header('referer');
        $domain = str_replace('http://', 'https://', $domain);

        //授权签名校验和授权是否有效判断
        if ((md5($domain . Config::get('salt')) !== substr($this->license[1], 32, 32)) || !$this->checkLicense($domain)) {
            $this->return['error']['message'] = 'Invalid license key';
            $this->return['error']['code']    = 105;
            return json($this->return);
        }

        $zip_path = $this->makeZip($domain);
        if (empty($zip_path)) {
            exit();
        }
        $size = filesize($zip_path);
        header("Content-type: application/octet-stream");
        header("Content-Disposition: attachment; filename=chevereto_" . $version . '.zip');
        header("Content-Length:" . $size);
        ob_clean();
        flush();
        // echo fread($zip_path, $size);
        readfile($zip_path);
        exit();

    }

    public function checkLicense($domain)
    {
        $domain = get_domain($domain);
        if (empty($domain)) {
            return false;
        }
        $domain_info = Loader::model('License')->getLicense($domain);
        if ($domain_info['expire_time'] < time()) {
            return false;
        }
        return true;
    }

    private function makeZip($domain)
    {
        $name = APP_PATH . '../runtime/temp/chevereto_' . md5($this->update['update_version_raw'] . '_' . $domain) . '.zip';
        if (is_file($name)) {
            return $name;
        }
        //创建临时文件夹和复制文件
        $temp_path = APP_PATH . "../runtime/temp/" . md5(time() . $domain);
        shell_exec("mkdir {$temp_path} && cp -rf {$this->update['down_path']}/* {$temp_path}/ && cd {$temp_path}/ && mv Nginx.txt nginx.conf && rm -rf *.url *.txt");
        $this->editKey($temp_path);
        $this->editApi($temp_path);
        mk_zip($name, $temp_path);
        return $name;
    }

    /**
     * 编辑Key文件
     * @param string $ext_path 解包后的目录(不含文件)
     *
     */
    private function editKey($ext_path)
    {
        $key_name = $ext_path . '/chevereto/app/license/key.php';
        $key_php  = file_get_contents($key_name);
        if ($key_php === false) {
            exit('无法打开授权信息文件!');
        }
        $key_php = explode(PHP_EOL, $key_php);
        foreach ($key_php as &$val) {
            if (strpos($val, '$license') !== false && strpos($val, ':') !== false && strpos($val, '\'') !== false && strpos($val, ';') !== false) {
                $domain = get_domain($this->request->header('Referer'));
                $domain = 'https://' . $domain;
                $key    = build_key($domain);
                $val    = "\$license = '{$key}';";
            }
            // $val .= PHP_EOL;
        }
        $key_php = implode(PHP_EOL, $key_php);
        return file_put_contents($key_name, $key_php);
    }

    /**
     * 编辑检查更新文件
     * @param string $ext_path 解包后的目录(不含文件)
     */
    private function editApi($ext_path)
    {
        $file_name = $ext_path . '/chevereto/app/lib/classes/class.settings.php';
        $api_php   = file_get_contents($file_name);
        if ($api_php === false) {
            exit('无法打开授权信息文件!');
        }
        $api_php = explode(PHP_EOL, $api_php);
        $domain  = Config::get('web_url');
        foreach ($api_php as &$val) {
            if (strpos($val, '$api') !== false && strpos($val, 'chevereto.com/api/') !== false) {
                $val = "		\$api = '{$domain}/api/';";
            } elseif (strpos($val, 'label') !== false && strpos($val, 'defined(') !== false) {
                $val = "                'label'=>'{$domain}',";
            } elseif (strpos($val, 'url') !== false && strpos($val, 'defined(') !== false) {
                $val = "                'url'=>'{$domain}',";
            }
            // $val .= PHP_EOL;
        }
        $api_php = implode(PHP_EOL, $api_php);
        return file_put_contents($file_name, $api_php);
    }
}
